"""Explore OAI-PMH user views
"""
from core_main_app.utils.rendering import render
from core_oaipmh_harvester_app.components.oai_record import api as oai_record_api


def data_detail(request):
    """ Display data's detail for OAI-PMH

    Args:
        request:

    Returns:

    """
    record_id = request.GET['id']
    data = dict()

    try:
        record = oai_record_api.get_by_id(record_id)
        template = record.harvester_metadata_format.template

        data = {'title': record.identifier, 'xml_content': record.xml_content,
                'template': {'id': template.id if template is not None else '',
                             'display_name': record.harvester_metadata_format.get_display_name(),
                             'hash': record.harvester_metadata_format.hash}}
    except Exception as e:
        error_message = "An error occured: {0}".format(str(e))
        return render(request,
                      'core_main_app/common/commons/error.html',
                      context={"error": error_message})

    context = {
        'data': data
    }

    assets = {
        "js": [
            {
                "path": 'core_main_app/common/js/XMLTree.js',
                "is_raw": False
            },
            {
                "path": 'core_main_app/user/js/data/detail.js',
                "is_raw": False
            },
        ],
        "css": ["core_main_app/common/css/XMLTree.css"],
    }

    return render(request, 'core_main_app/user/data/detail.html', context=context, assets=assets)



""" OaiPmh Query api
"""
import core_explore_common_app.components.query.api as api_query
from core_explore_common_app.components.query.api import upsert


def add_oaipmh_data_source(query, data_source):
    """ Add an oaipmh data source to the query

    Args:
        query:
        data_source:

    Returns:

    """
    # the unique identifier for an oaipmh data source is its instance_id
    # because there is no constraint on the name
    data_source_found = False
    for data_source_item in query.data_sources:
        if 'instance_id' in data_source_item.query_options \
                and data_source_item.query_options['instance_id'] == data_source.query_options['instance_id']:
            data_source_found = True

    if not data_source_found:
        # add data source to query if not present
        query.data_sources.append(data_source)
        # update query
        return upsert(query)


def remove_oaipmh_data_source(query, instance_id):
    """ Remove an oaipmh data source to the query

    Args:
        query:
        instance_id:

    Returns:

    """
    data_source = None
    for data_source_item in query.data_sources:
        if 'instance_id' in data_source_item.query_options \
                and data_source_item.query_options['instance_id'] == instance_id:
            data_source = data_source_item

    if data_source:
        return api_query.remove_data_source(query, data_source)



"""Url router for the REST API
"""
from django.conf.urls import url

from core_explore_oaipmh_app.rest.query import views as query_views
from core_explore_oaipmh_app.rest.result import views as result_views

urlpatterns = [
    url(r'^execute-query', query_views.ExecuteQueryView.as_view(),
        name='core_explore_oaipmh_rest_execute_query'),
    url(r'^result', result_views.get_result_from_data_id,
        name='core_explore_oaipmh_app_rest_get_result_from_data_id')

]



""" Permissions Test for OAI Explore Query Rest API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status
from rest_framework.response import Response

from core_explore_oaipmh_app.rest.query import views as query_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_oaipmh_harvester_app.rest.oai_record.abstract_views import AbstractExecuteQueryView


class TestGetExecuteQueryRegistry(SimpleTestCase):
    def setUp(self):
        super(TestGetExecuteQueryRegistry, self).setUp()
        self.one_record_data = {"query": "{"
                                         "\"experiment.experimentType.tracerDiffusivity.material.materialName\": \"Test 1\"}"}
        self.user = create_mock_user('1')

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_execute_query):
        # Arrange
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(query_views.ExecuteQueryView.as_view(), None, data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_authenticated_returns_http_200(self, mock_execute_query):
        # Arrange
        user = create_mock_user('1')
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(query_views.ExecuteQueryView.as_view(), user=user,
                                              data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_execute_query):
        # Arrange
        user = create_mock_user('1', is_staff=True)
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_get(query_views.ExecuteQueryView.as_view(), user=user,
                                              data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostExecuteQueryRegistry(SimpleTestCase):
    def setUp(self):
        super(TestPostExecuteQueryRegistry, self).setUp()
        self.one_record_data = {"query": "{"
                                         "\"experiment.experimentType.tracerDiffusivity.material.materialName\": \"Test 1\"}"}
        self.user = create_mock_user('1')

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_anonymous_returns_http_200(self, mock_execute_query):
        # Arrange
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_post(query_views.ExecuteQueryView.as_view(), None, data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_authenticated_returns_http_200(self, mock_execute_query):
        # Arrange
        user = create_mock_user('1')
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_post(query_views.ExecuteQueryView.as_view(), user=user,
                                              data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(AbstractExecuteQueryView, "execute_query")
    def test_staff_returns_http_200(self, mock_execute_query):
        # Arrange
        user = create_mock_user('1', is_staff=True)
        data = self.one_record_data
        mock_execute_query.return_value = Response(status=status.HTTP_200_OK)

        # Act
        response = RequestMock.do_request_post(query_views.ExecuteQueryView.as_view(), user=user,
                                              data=data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Url router for the explore OAI-PMH search application
"""
from django.conf.urls import url, include

import core_explore_oaipmh_app.views.user.ajax as user_ajax
import core_explore_oaipmh_app.views.user.views as user_views

urlpatterns = [
    url(r'^get_data_sources', user_ajax.get_data_source_list_oaipmh,
        name='core_explore_oaipmh_app_get_data_sources'),
    url(r'^update_data_sources', user_ajax.update_data_source_list_oaipmh,
        name='core_explore_oaipmh_app_update_data_sources'),
    url(r'^data', user_views.data_detail,
        name='core_explore_oaipmh_app_data_detail'),
    url(r'^rest/', include('core_explore_oaipmh_app.rest.urls')),
]



""" Permissions Test for OAI Explore Result Rest API
"""
from django.test import SimpleTestCase
from mock.mock import patch, Mock
from rest_framework import status

from core_explore_common_app.rest.result.serializers import ResultSerializer
from core_explore_oaipmh_app.rest.result import views as result_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_oaipmh_harvester_app.components.oai_record import api as oai_record_api
from core_oaipmh_harvester_app.components.oai_record.models import OaiRecord


class TestGetResultQueryRegistry(SimpleTestCase):
    def setUp(self):
        super(TestGetResultQueryRegistry, self).setUp()
        self.data = {"id": 1}

    @patch.object(ResultSerializer, "data")
    @patch.object(oai_record_api, "get_by_id")
    def test_anonymous_returns_http_200(self, mock_oai_record_api_get_by_id, mock_serializer_data):
        # Arrange
        mock_oai_record_api_get_by_id.return_value = Mock(spec=OaiRecord)
        mock_serializer_data.return_value = Mock(spec=ResultSerializer)

        # Act
        response = RequestMock.do_request_get(result_views.get_result_from_data_id, None, data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(ResultSerializer, "data")
    @patch.object(oai_record_api, "get_by_id")
    def test_authenticated_returns_http_200(self, mock_oai_record_api_get_by_id, mock_serializer_data):
        # Arrange
        user = create_mock_user('1')
        mock_oai_record_api_get_by_id.return_value = Mock(spec=OaiRecord)
        mock_serializer_data.return_value = Mock(spec=ResultSerializer)

        # Act
        response = RequestMock.do_request_get(result_views.get_result_from_data_id, user=user, data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(ResultSerializer, "data")
    @patch.object(oai_record_api, "get_by_id")
    def test_staff_returns_http_200(self, mock_oai_record_api_get_by_id, mock_serializer_data):
        # Arrange
        user = create_mock_user('1', is_staff=True)
        mock_oai_record_api_get_by_id.return_value = Mock(spec=OaiRecord)
        mock_serializer_data.return_value = Mock(spec=ResultSerializer)

        # Act
        response = RequestMock.do_request_get(result_views.get_result_from_data_id, user=user, data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
